
//
// PathExpander.cs
//
// Author:
//   Tomas Restrepo (tomasr@mvps.org)
//


using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;

using Microsoft.BizTalk.Message.Interop;


namespace Winterdom.BizTalk.Pipelines.Tracer
{
   /// <summary>
   /// Utility class used to expand variables
   /// in path names using the value of
   /// message context properties.
   /// </summary>
   /// <remarks>
   /// We recognize the following structures:
   ///   <list type="bullet">
   ///      <item>%MessageID%</item>
   ///      <item>%InboundTransportType%</item>
   ///      <item>%OutboundTransportType%</item>
   ///      <item>%ReceivePortName%</item>
   ///      <item>%InterchangeID%</item>
   ///      <item>%datetime%</item>
   ///      <item>%datetime.tz%</item>
   ///      <item>%time%</item>
   ///      <item>%time.tz%</item>
   ///   </list>
   /// </remarks>
   internal class PathExpander
   {
      private StringBuilder _path; 
      private IBaseMessage _msg;

      /// <summary>
      /// Create a new instance
      /// </summary>
      /// <param name="basePath">Base path to expand</param>
      /// <param name="msg">Message to read from</param>
      public PathExpander(string basePath, IBaseMessage msg)
      {
         if ( basePath == null )
            throw new ArgumentNullException("basePath");
         if ( msg == null )
            throw new ArgumentNullException("msg");

         _path = new StringBuilder(basePath);
         _msg = msg;
      }

      /// <summary>
      /// Expand the message variables. 
      /// </summary>
      /// <returns>The expanded path</returns>
      public string ExpandPath()
      {
         ExpandMessageID();
         ExpandInterchangeID();
         ExpandInboundTransportType();
         ExpandOutboundTransportType();
         ExpandReceivePortName();
         ExpandDateTime();
         ExpandDateTimeTZ();
         ExpandTime();
         ExpandTimeTZ();

         return _path.ToString();
      }

      private void ExpandMessageID()
      {
         Guid msgID = _msg.MessageID;
         _path.Replace("%"+MacroNames.MessageID+"%", msgID.ToString("B"));
      }

      private void ExpandInboundTransportType()
      {
         string val = (string)
            GetContextProp(MacroNames.InboundTransportType, ContextPropNS.BTS);
         if ( val != null ) {
            _path.Replace("%"+MacroNames.InboundTransportType+"%", val);
         }
      }

      private void ExpandOutboundTransportType()
      {
         string val = (string)
            GetContextProp(MacroNames.OutboundTransportType, ContextPropNS.BTS);
         if ( val != null ) {
            _path.Replace("%"+MacroNames.OutboundTransportType+"%", val);
         }
      }

      private void ExpandReceivePortName()
      {
         string val = (string)
            GetContextProp(MacroNames.ReceivePortName, ContextPropNS.BTS);
         if ( val != null ) {
            _path.Replace("%"+MacroNames.ReceivePortName+"%", val);
         }
      }

      private void ExpandInterchangeID()
      {
         string val = (string)
            GetContextProp(MacroNames.InterchangeID, ContextPropNS.BTS);
         if ( val != null ) {
            _path.Replace("%"+MacroNames.InterchangeID+"%", val);
         }
      }

      private void ExpandDateTime()
      {
         string val = 
            DateTime.Now.ToUniversalTime().ToString("yyyy-MM-ddTHHmmss");
         _path.Replace("%"+MacroNames.DateTime+"%", val);
      }
      
      private void ExpandDateTimeTZ()
      {
         string val = 
            DateTime.Now.ToString("yyyy-MM-ddTHHmmsszzz");
         _path.Replace("%"+MacroNames.DateTimeTZ+"%", val.Replace(":", ""));
      }

      private void ExpandTime()
      {
         string val = 
            DateTime.Now.ToString("HHmmss");
         _path.Replace("%"+MacroNames.Time+"%", val);
      }

      private void ExpandTimeTZ()
      {
         string val = 
            DateTime.Now.ToString("HHmmsszzz");
         _path.Replace("%"+MacroNames.TimeTZ+"%", val.Replace(":", ""));
      }

      /// <summary>
      /// Extracts a property 
      /// from the message context
      /// </summary>
      /// <param name="name">Name of the property</param>
      /// <param name="ns">Namespace of the property</param>
      /// <returns>Property value or null</returns>
      private object GetContextProp(string name, string ns)
      {
         return _msg.Context.Read(name, ns);
      }

   } // class PathExpander

} // namespace Winterdom.BizTalk.Pipelines.Tracer
