
//
// TraceStream.cs
//
// Author:
//   Tomas Restrepo (tomasr@mvps.org)
//


using System;
using System.Diagnostics;
using System.IO;


namespace Winterdom.BizTalk.Pipelines.Tracer
{
   /// <summary>
   /// Stream class that traces the message to 
   /// a secondary stream as the message is read.
   /// </summary>
   public class TraceStream : Stream, IDisposable
   {
      private Stream _origStream;
      private Stream _dumpStream;


      #region Properties
      // 
      // Properties
      //

      /// <summary>
      /// Returns whether the stream can be read
      /// </summary>
      public override bool CanRead  
      {
         get { return _origStream.CanRead; }
      }

      /// <summary>
      /// Returns whether the stream can be written to
      /// </summary>
      public override bool CanWrite 
      {
         get { return _origStream.CanWrite; }
      }

      /// <summary>
      /// Returns whether the stream is seekable
      /// </summary>
      public override bool CanSeek 
      {
         get { return _origStream.CanSeek; }
      }

      /// <summary>
      /// Returns the length of the stream
      /// </summary>
      public override long Length 
      {
         get { return _origStream.Length; }
      }

      /// <summary>
      /// Returns the current position of the stream
      /// </summary>
      public override long Position 
      {
         get { return _origStream.Position; }
         set { Seek(value, SeekOrigin.Begin); }
      }

      #endregion // Properties


      /// <summary>
      /// Creates a new instance
      /// </summary>
      /// <param name="origStream">Stream to read from</param>
      /// <param name="dumpStream">Stream to write to</param>
      public TraceStream(Stream origStream, Stream dumpStream)
      {
         if ( origStream == null ) 
            throw new ArgumentNullException("origStream");
         if ( dumpStream == null ) 
            throw new ArgumentNullException("dumpStream");
         
         _origStream = origStream;
         _dumpStream = dumpStream;
      }

      #region Overriden Methods
      //
      // Overriden Methods
      //

      /// <summary>
      /// Close the stream
      /// </summary>
      public override void Close()
      {
         Dispose(true);
      }

      /// <summary>
      /// Flush the stream
      /// </summary>
      public override void Flush()
      {
         _origStream.Flush();
         if ( _dumpStream != null ) {
            _dumpStream.Flush();
         }
      }

      /// <summary>
      /// Seek the streams
      /// </summary>
      /// <param name="offset">Number of bytes to move</param>
      /// <param name="origin">From where to move</param>
      /// <returns>The new position</returns>
      public override long Seek(long offset, SeekOrigin origin)
      {
         // 
         // Heuristic: We know that the pipeline reads the
         // stream multiple times, but we also know that it uses
         // Seek to bring it back to the beginning 
         // every time. So, once we go to the start, close it
         //

         if ( _dumpStream != null
            && origin == SeekOrigin.Begin
            && offset == 0 )
         {
            DisposeDumpStream();
         }
         return _origStream.Seek(offset, origin);
      }

      public override void SetLength(long value)
      {
         _origStream.SetLength(value);
         // ain't probably do what we expect...
         if ( _dumpStream != null ) {
            _dumpStream.SetLength(value);
         }
      }



      /// <summary>
      /// Read a byte from the original stream,
      /// writing it to the dumpStream
      /// </summary>
      /// <returns>the byte or -1</returns>
      public override int ReadByte()
      {
         int b = _origStream.ReadByte();
         if ( b != -1 ) 
         {
            if ( _dumpStream != null ) {
               _dumpStream.WriteByte((byte)b);
            }
         } 
         else 
         {
            DisposeDumpStream();
         }
         return b;
      }

      /// <summary>
      /// Read bytes from the original stream,
      /// writing them to the dumpStream
      /// </summary>
      /// <param name="buffer">Buffer to read into</param>
      /// <param name="offset">Offset to write into the buffer</param>
      /// <param name="count">Number of bytes to read</param>
      /// <returns>Number of bytes read, or -1</returns>
      public override int Read(byte[] buffer, int offset, int count)
      {
         int v = _origStream.Read(buffer, offset, count);
         if ( v > 0 ) 
         {
            if ( _dumpStream != null ) {
               _dumpStream.Write(buffer, offset, v);
            }
         } 
         else 
         {
            DisposeDumpStream();
         }
         return v;
      }

      /// <summary>
      /// Write a byte to the original and dump streams
      /// </summary>
      /// <param name="value">Byte to write</param>
      public override void WriteByte(byte value)
      {
         _origStream.WriteByte(value);

         if ( _dumpStream != null ) {
            _dumpStream.WriteByte(value);
         }
      }

      /// <summary>
      /// Write an array of bytes to the original 
      /// and dump streams
      /// </summary>
      /// <param name="buffer">Buffer to write</param>
      /// <param name="offset">Offset to start writing</param>
      /// <param name="count">Number of bytes to write</param>
      public override void Write(byte[] buffer, int offset, int count)
      {
         _origStream.Write(buffer, offset, count);

         if ( _dumpStream != null ) {
            _dumpStream.Write(buffer, offset, count);
         }
      }


      #endregion // Overriden Methods


      /// <summary>
      /// Dispose of this instance
      /// </summary>
      /// <param name="disposing">True if disposing, false otherwise</param>
      protected virtual void Dispose(bool disposing)
      {
         if ( disposing )
         {
            if ( _origStream != null ) {
               _origStream.Close();
               _origStream = null;
            }
            DisposeDumpStream();
            GC.SuppressFinalize(this);
         }
      }

      /// <summary>
      /// Dispose of the dump stream we use
      /// </summary>
      private void DisposeDumpStream()
      {
         if ( _dumpStream != null ) {
            _dumpStream.Flush();
            _dumpStream.Close();
            _dumpStream = null;
         }
      }

      #region IDisposable Implementation
      //
      // IDisposable Implementation
      //

      void IDisposable.Dispose()
      {
         Dispose(true);
      }

      #endregion // IDisposable Implementation


   } // class TraceStream

} // namespace Winterdom.BizTalk.Pipelines.Tracer
