
//
// TracerComponent.cs
//
// Author:
//   Tomas Restrepo (tomasr@mvps.org)
//


using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Resources;
using System.Runtime.InteropServices;
using System.Text;

using Microsoft.BizTalk.Component.Interop;
using Microsoft.BizTalk.Message.Interop;


namespace Winterdom.BizTalk.Pipelines.Tracer
{

   /// <summary>
   /// Pipeline Component for BizTalk Server 2004
   /// which can trace messages going through and save them
   /// into files.
   /// </summary>
   [ ComponentCategory(CategoryTypes.CATID_PipelineComponent) ]
   [ ComponentCategory(CategoryTypes.CATID_Any) ]
   [ ComponentCategory(CategoryTypes.CATID_Validate) ]
   [ Guid("40ded55c-eef9-4a6d-8aa1-e7ddf2df9711") ]
   public class TracerComponent : 
      IBaseComponent, 
      Microsoft.BizTalk.Component.Interop.IComponent, 
      IComponentUI, 
      IPersistPropertyBag
   {
      private const string VERSION = "1.0";
      private readonly Guid CLSID = new Guid("40ded55c-eef9-4a6d-8aa1-e7ddf2df9711");

      private string _tracePath;

      #region Properties
      //
      // Properties
      //

      /// <summary>
      /// Path to store the trace files to.
      /// </summary>
      [ TracerDescription("TracerPathPropDesc") ]
      public string TracePath {
         get { return _tracePath; }
         set { _tracePath = value; }
      }

      #endregion // Properties


      #region IBaseComponent Members

      //
      // IBaseComponent Members
      // 

      /// <summary>
      /// Description of the component
      /// </summary>
      [ Browsable(false) ]
      public string Description {
         get { return TracerResources.TracerName; }
      }

      /// <summary>
      /// Name of the component
      /// </summary>
      [ Browsable(false) ]
      public string Name {
         get { return TracerResources.TracerName; }
      }

      /// <summary>
      /// Version of the component
      /// </summary>
      [ Browsable(false) ]
      public string Version {
         get { return VERSION; }
      }

      #endregion // IBaseComponent Members


      #region IComponent Members

      //
      // IComponent Members
      // 

      /// <summary>
      /// Execute the pipeline component
      /// </summary>
      /// <param name="context"></param>
      /// <param name="msg"></param>
      /// <returns></returns>
      public IBaseMessage Execute(IPipelineContext context, IBaseMessage msg)
      {
         //
         // Expand the Trace Path variables,
         // then create the file and trace to it
         //
         try 
         {
            PathExpander expander = new PathExpander(_tracePath, msg);
            string filename = expander.ExpandPath();
            Stream dumpStream = new FileStream(filename, FileMode.OpenOrCreate);
            Stream origStream = msg.BodyPart.GetOriginalDataStream();
            TraceStream traceStream = new TraceStream(origStream, dumpStream);

            msg.BodyPart.Data = traceStream;
            return msg;

         } catch ( Exception ex ) {
            string error = 
               string.Format(TracerResources.ErrorCreateFile, ex.Message);
            ApplicationException newEx 
               = new ApplicationException(error, ex);
            newEx.Source = this.Name;

            throw newEx;
         }
      }

      #endregion // IComponent Members


      #region IComponentUI Members
      //
      // IComponentUI Members
      //

      /// <summary>
      /// Icon to present to the user
      /// </summary>
      [ Browsable(false) ]
      public IntPtr Icon {
         get { return LoadIcon(); }
      }

      
      /// <summary>
      /// Validate the configuration to see if it's valid.
      /// Not implemented yet.
      /// </summary>
      /// <param name="projectSystem"></param>
      /// <returns></returns>
      public IEnumerator Validate(object projectSystem)
      {
         return null;
      }


      #endregion // IComponentUI Members


      #region IPersistPropertyBag Members
      //
      // IPersistPropertyBag Members
      //

      /// <summary>
      /// Not necessary, Not implemented
      /// </summary>
      public void InitNew()
      {
      }

      /// <summary>
      /// Get the component Class ID
      /// </summary>
      /// <param name="classID">Class ID of this component</param>
      public void GetClassID(out Guid classID)
      {
         classID = CLSID;
      }

      public void Load(IPropertyBag propertyBag, int errorLog)
      {
         object value;
         try {
            propertyBag.Read("TracePath", out value, 0);
            _tracePath = (string)value;

         } catch ( ArgumentException ) {
            _tracePath = null;
         }
      }

      public void Save(IPropertyBag propertyBag, bool clearDirty, bool saveAllProperties)
      {
         object value = _tracePath;
         propertyBag.Write("TracePath", ref value);
      }

      #endregion // IPersistPropertyBag Members


      #region Private Methods
      //
      // Private Methods
      //

      /// <summary>
      /// Load this component's icon for the
      /// toolbox and return a pointer to it.
      /// </summary>
      /// <returns>A pointer to the icon</returns>
      private IntPtr LoadIcon()
      {
         return TracerResources.TracerIcon.GetHicon();
      }

      #endregion // Private Methods


   } // class TracerComponent

} // namespace Winterdom.BizTalk.Pipelines.Tracer